import pytest

from .rocketchat import add_rocketchat_options, rocketchat_send_message


def pytest_addoption(parser):
    add_rocketchat_options(parser)


class TestResult:
    failed: int
    passed: int
    skipped: int
    error: int
    xfailed: int
    xpassed: int


@pytest.hookimpl(hookwrapper=True)
def pytest_terminal_summary(terminalreporter, exitstatus, config):
    yield
    # special check for pytest-xdist plugin, cause we do not want to send report for each worker.
    if hasattr(terminalreporter.config, "workerinput"):
        return
    test_result = TestResult()
    test_result.failed = len(terminalreporter.stats.get("failed", []))
    test_result.passed = len(terminalreporter.stats.get("passed", []))
    test_result.skipped = len(terminalreporter.stats.get("skipped", []))
    test_result.error = len(terminalreporter.stats.get("error", []))
    test_result.xfailed = len(terminalreporter.stats.get("xfailed", []))
    test_result.xpassed = len(terminalreporter.stats.get("xpassed", []))
    if config.option.rocket_url and config.option.rocket_password:
        rocketchat_send_message(test_result, config, exitstatus)



from rocketchat_API.rocketchat import RocketChat


def add_rocketchat_options(parser):
    group = parser.getgroup("rocketchat")
    group.addoption(
        "--ssl_verify",
        action="store",
        dest="ssl_verify",
        default=True,
        help="Set the TLS certificate verification",
    )
    group.addoption(
        "--rocketchat_server_url",
        action="store",
        dest="rocket_url",
        default=None,
        help="Set rocketchat server url",
    )
    group.addoption(
        "--rocketchat_channel",
        action="store",
        dest="rocket_channel",
        default=None,
        help="Set the channel name to report",
    )
    group.addoption(
        "--rocketchat_username",
        action="store",
        dest="rocket_username",
        default=None,
        help="Set the reporter name",
    )
    group.addoption(
        "--rocketchat_password",
        action="store",
        dest="rocket_password",
        default=None,
        help="Used for authentication to rocketchat by rocket_username",
    )

    group.addoption(
        "--rocketchat_report_link",
        action="store",
        dest="rocket_report_link",
        default=None,
        help="Set the report link",
    )

    group.addoption(
        "--rocketchat_message_prefix",
        action="store",
        dest="rocket_message_prefix",
        default=None,
        help="Set a prefix to come before the test result counts.",
    )

    group.addoption(
        "--rocketchat_timeout",
        action="store",
        dest="rocket_timeout",
        default=10,
        help="Set the report send timeout",
    )

    group.addoption(
        "--rocketchat_success_emoji",
        action="store",
        dest="rocket_success_emoji",
        default=":thumbsup:",
        help="Set emoji for a successful run",
    )

    group.addoption(
        "--rocketchat_failed_emoji",
        action="store",
        dest="rocket_failed_emoji",
        default=":thumbsdown:",
        help="Set emoji for a failed run",
    )


def rocketchat_send_message(test_result, config, exitstatus):
    timeout = config.option.rocket_timeout
    report_link = config.option.rocket_report_link
    server_url = config.option.rocket_url
    channel = config.option.rocket_channel
    ssl_verify = config.option.ssl_verify
    message_prefix = config.option.rocket_message_prefix
    rocket_username = (
        config.option.rocket_username
        if config.option.rocket_username
        else "Regression testing results"
    )
    rocket_pass = config.option.rocket_password
    if int(exitstatus) == 0:
        color = "#56a64f"
        emoji = config.option.rocket_success_emoji
    else:
        color = "#ff0000"
        emoji = config.option.rocket_failed_emoji
    final_results = "Passed=%s Failed=%s Skipped=%s Error=%s XFailed=%s XPassed=%s" % (
        test_result.passed,
        test_result.failed,
        test_result.skipped,
        test_result.error,
        test_result.xfailed,
        test_result.xpassed,
    )
    if report_link:
        final_results = "<%s|%s>" % (report_link, final_results)
    if message_prefix:
        final_results = "%s: %s" % (message_prefix, final_results)

    # # uncomment after fix 3.11.1 rocket issue
    # # https://github.com/RocketChat/Rocket.Chat/issues/20556
    #
    # results_pattern = {
    #     "color": color,
    #     "text": final_results,
    #     "mrkdwn_in": [
    #         "text",
    #         "pretext",
    #     ]
    # }

    rocket_client = RocketChat(
        user=rocket_username,
        password=rocket_pass,
        server_url=server_url,
        ssl_verify=ssl_verify,
    )

    rocket_client.chat_post_message(
        # attachments=[results_pattern],  # uncomment after fix3.11.1 rocket
        channel=channel,
        alias=rocket_username,
        emoji=emoji,
        text=final_results,
        timeout=timeout,
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pytest_rocketchat']

package_data = \
{'': ['*']}

install_requires = \
['rocketchat-API']

entry_points = \
{'pytest11': ['pytest-rocketchat = pytest_rocketchat.plugin']}

setup_kwargs = {
    'name': 'pytest-rocketchat',
    'version': '1.0.1',
    'description': 'Pytest to Rocket.Chat reporting plugin',
    'long_description': '# pytest-rocketchat\nPytest to [RocketChat](https://github.com/RocketChat) reporting plugin.\n\n___Inspired by [pytest-slack](https://pypi.org/project/pytest-slack/) & [pytest-messenger](https://pypi.org/project/pytest-messenger/).___\n\n## Usage\n```\n$ pytest --rocketchat_server_url=https://your.chat --rocketchat_username=username_or_a_botname --rocketchat_password=passowrord_of_user_or_of_a_bot\n```\nOptions:\n- --rocketchat_server_url* (Required)\n- --rocketchat_username* (Required)\n- --rocketchat_password* (Required)\n- --rocketchat_report_link\n- --rocketchat_message_prefix\n- --rocketchat_timeout\n- --rocketchat_success_emoji\n- --rocketchat_failed_emoji\n- --ssl_verify\n\n## Requirements\n- Python >= 3.6\n\n## Installation\nYou can install "pytest-rocketchat" via [pip](https://pypi.python.org/pypi/pip/):\n```\n$ pip install pytest-rocketchat\n```\nIf you encounter any problems, please file an [issue](https://github.com/aleksandr-kotlyar/pytest-rocketchat/issues) along with a detailed description.\n',
    'author': 'Aleksandr Kotlyar',
    'author_email': 'ask.kotlyar@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'http://github.com/aleksandr-kotlyar/pytest-rocketchat',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
